@@ -83,6 +83,10 @@ extern "C" {
 //deflate max ratio is about 1000
 #define HTP_COMPRESSION_BOMB_RATIO          2048
 #define HTP_COMPRESSION_BOMB_LIMIT          1048576
+// 0.1 second
+#define HTP_COMPRESSION_TIME_LIMIT_USEC     100000
+// test time for compression every 256 callbacks
+#define HTP_COMPRESSION_TIME_FREQ_TEST      256
 
 #define HTP_FIELD_LIMIT_HARD               18000
 #define HTP_FIELD_LIMIT_SOFT               9000
@@ -157,7 +161,7 @@ int htp_is_line_whitespace(unsigned char *data, size_t len);
 
 int htp_connp_is_line_folded(unsigned char *data, size_t len);
 int htp_is_folding_char(int c);
-int htp_connp_is_line_terminator(htp_connp_t *connp, unsigned char *data, size_t len);
+int htp_connp_is_line_terminator(htp_connp_t *connp, unsigned char *data, size_t len, int next_no_lf);
 int htp_connp_is_line_ignorable(htp_connp_t *connp, unsigned char *data, size_t len);
 
 int htp_parse_uri(bstr *input, htp_uri_t **uri);
@@ -167,8 +171,6 @@ int htp_validate_hostname(bstr *hostname);
 int htp_parse_uri_hostport(htp_connp_t *connp, bstr *input, htp_uri_t *uri);
 int htp_normalize_parsed_uri(htp_tx_t *tx, htp_uri_t *parsed_uri_incomplete, htp_uri_t *parsed_uri);
 bstr *htp_normalize_hostname_inplace(bstr *input);
-void htp_replace_hostname(htp_connp_t *connp, htp_uri_t *parsed_uri, bstr *hostname);
-int htp_is_uri_unreserved(unsigned char c);
 
 int htp_decode_path_inplace(htp_tx_t *tx, bstr *path);
 
