@@ -775,6 +775,10 @@ htp_status_t htp_connp_REQ_LINE_complete(htp_connp_t *connp) {
     #ifdef HTP_DEBUG
     fprint_raw_data(stderr, __func__, data, len);
     #endif
+    if (len == 0) {
+        htp_connp_req_clear_buffer(connp);
+        return HTP_DATA;
+    }
 
     // Is this a line that should be ignored?
     if (htp_connp_is_line_ignorable(connp, data, len)) {
@@ -839,11 +843,11 @@ htp_status_t htp_connp_REQ_FINALIZE(htp_connp_t *connp) {
         }
         if (connp->in_next_byte != LF || connp->in_current_consume_offset >= connp->in_current_read_offset) {
             for (;;) {//;i < max_read; i++) {
-                IN_COPY_BYTE_OR_RETURN(connp);
-                // Have we reached the end of the line? For some reason
-                // we can't test after IN_COPY_BYTE_OR_RETURN */
+                // peek until LF but do not mark it read so that REQ_LINE works
+                IN_PEEK_NEXT(connp);
                 if (connp->in_next_byte == LF)
                     break;
+                IN_COPY_BYTE_OR_RETURN(connp);
             }
         }
     }
@@ -873,37 +877,34 @@ htp_status_t htp_connp_REQ_FINALIZE(htp_connp_t *connp) {
     while ((pos < len) && (!htp_is_space(data[pos])))
         pos++;
 
-    if (pos <= mstart) {
-        //empty whitespace line
-        htp_status_t rc = htp_tx_req_process_body_data_ex(connp->in_tx, data, len);
-        htp_connp_req_clear_buffer(connp);
-        return rc;
-    } else {
+    if (pos > mstart) {
+        //non empty whitespace line
         int methodi = HTP_M_UNKNOWN;
         bstr *method = bstr_dup_mem(data + mstart, pos - mstart);
         if (method) {
             methodi = htp_convert_method_to_number(method);
             bstr_free(method);
         }
-        if (methodi == HTP_M_UNKNOWN) {
-            // Interpret remaining bytes as body data
-            htp_log(connp, HTP_LOG_MARK, HTP_LOG_WARNING, 0, "Unexpected request body");
-            htp_status_t rc = htp_tx_req_process_body_data_ex(connp->in_tx, data, len);
-            htp_connp_req_clear_buffer(connp);
-            return rc;
+        if (methodi != HTP_M_UNKNOWN) {
+            connp->in_body_data_left = -1;
+            return htp_tx_state_request_complete(connp->in_tx);
         } // else continue
+        if (connp->in_body_data_left <= 0) {
+            // log only once per transaction
+            htp_log(connp, HTP_LOG_MARK, HTP_LOG_WARNING, 0, "Unexpected request body");
+        } else {
+            connp->in_body_data_left = 1;
+        }
     }
-    //unread last end of line so that REQ_LINE works
-    if (connp->in_current_read_offset < (int64_t)len) {
-        connp->in_current_read_offset=0;
-    } else {
-        connp->in_current_read_offset-=len;
+    //Adds linefeed to the buffer if there was one
+    if (connp->in_next_byte == LF) {
+        IN_COPY_BYTE_OR_RETURN(connp);
+        htp_connp_req_consolidate_data(connp, &data, &len);
     }
-    if (connp->in_current_read_offset < connp->in_current_consume_offset) {
-        connp->in_current_consume_offset=connp->in_current_read_offset;
-    }
-
-    return htp_tx_state_request_complete(connp->in_tx);
+    // Interpret remaining bytes as body data
+    htp_status_t rc = htp_tx_req_process_body_data_ex(connp->in_tx, data, len);
+    htp_connp_req_clear_buffer(connp);
+    return rc;
 }
 
 htp_status_t htp_connp_REQ_IGNORE_DATA_AFTER_HTTP_0_9(htp_connp_t *connp) {
@@ -992,7 +993,7 @@ int htp_connp_req_data(htp_connp_t *connp, const htp_time_t *timestamp, const vo
     // only if the stream has been closed. We do not allow zero-sized
     // chunks in the API, but we use them internally to force the parsers
     // to finalize parsing.
-    if (((data == NULL) || (len == 0)) && (connp->in_status != HTP_STREAM_CLOSED)) {
+    if (len == 0 && connp->in_status != HTP_STREAM_CLOSED) {
         htp_log(connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0, "Zero-length data chunks are not allowed");
 
         #ifdef HTP_DEBUG
@@ -1045,7 +1046,25 @@ int htp_connp_req_data(htp_connp_t *connp, const htp_time_t *timestamp, const vo
 
         // Return if there's been an error or if we've run out of data. We are relying
         // on processors to supply error messages, so we'll keep quiet here.
-        htp_status_t rc = connp->in_state(connp);
+
+        htp_status_t rc;
+        //handle gap
+        if (data == NULL && len > 0) {
+            //cannot switch over a function pointer in C
+            if (connp->in_state == htp_connp_REQ_BODY_IDENTITY ||
+                connp->in_state == htp_connp_REQ_IGNORE_DATA_AFTER_HTTP_0_9) {
+                rc = connp->in_state(connp);
+            } else if (connp->in_state == htp_connp_REQ_FINALIZE) {
+                //simple version without probing
+                rc = htp_tx_state_request_complete(connp->in_tx);
+            } else {
+                // go to htp_connp_REQ_CONNECT_PROBE_DATA ?
+                htp_log(connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0, "Gaps are not allowed during this state");
+                return HTP_STREAM_CLOSED;
+            }
+        } else {
+            rc = connp->in_state(connp);
+        }
         if (rc == HTP_OK) {
             if (connp->in_status == HTP_STREAM_TUNNEL) {
                 #ifdef HTP_DEBUG
