@@ -342,6 +342,20 @@ htp_status_t htp_connp_RES_BODY_CHUNKED_DATA(htp_connp_t *connp) {
     return HTP_DATA;
 }
 
+static inline int is_chunked_ctl_char(const unsigned char c) {
+    switch (c) {
+        case 0x0d:
+        case 0x0a:
+        case 0x20:
+        case 0x09:
+        case 0x0b:
+        case 0x0c:
+            return 1;
+        default:
+            return 0;
+    }
+}
+
 /**
  * Peeks ahead into the data to try to see if it starts with a valid Chunked
  * length field.
@@ -361,7 +375,7 @@ static inline int data_probe_chunk_length(htp_connp_t *connp) {
     while (i < len) {
         unsigned char c = data[i];
 
-        if (c == 0x0d || c == 0x0a || c == 0x20 || c == 0x09 || c == 0x0b || c == 0x0c) {
+        if (is_chunked_ctl_char(c)) {
             // ctl char, still good.
         } else if (isdigit(c) || (c >= 'a' && c <= 'f') || (c >= 'A' && c <= 'F')) {
             // real chunklen char
@@ -386,7 +400,8 @@ htp_status_t htp_connp_RES_BODY_CHUNKED_LENGTH(htp_connp_t *connp) {
         OUT_COPY_BYTE_OR_RETURN(connp);
 
         // Have we reached the end of the line? Or is this not chunked after all?
-        if (connp->out_next_byte == LF || !data_probe_chunk_length(connp)) {
+        if (connp->out_next_byte == LF ||
+                (!is_chunked_ctl_char(connp->out_next_byte) && !data_probe_chunk_length(connp))) {
             unsigned char *data;
             size_t len;
 
@@ -545,11 +560,13 @@ htp_status_t htp_connp_RES_BODY_DETERMINE(htp_connp_t *connp) {
             return rc;
         } else if (connp->out_tx->response_status_number == 407) {
             // proxy telling us to auth
-            connp->in_status = HTP_STREAM_DATA;
+            if (connp->in_status != HTP_STREAM_ERROR)
+                connp->in_status = HTP_STREAM_DATA;
         } else {
             // This is a failed CONNECT stream, which means that
             // we can unblock request parsing
-            connp->in_status = HTP_STREAM_DATA;
+            if (connp->in_status != HTP_STREAM_ERROR)
+                connp->in_status = HTP_STREAM_DATA;
 
             // We are going to continue processing this transaction,
             // adding a note for ourselves to stop at the end (because
@@ -571,7 +588,8 @@ htp_status_t htp_connp_RES_BODY_DETERMINE(htp_connp_t *connp) {
         if (te == NULL && cl == NULL) {
             connp->out_state = htp_connp_RES_FINALIZE;
 
-            connp->in_status = HTP_STREAM_TUNNEL;
+            if (connp->in_status != HTP_STREAM_ERROR)
+                connp->in_status = HTP_STREAM_TUNNEL;
             connp->out_status = HTP_STREAM_TUNNEL;
 
             // we may have response headers
@@ -608,6 +626,19 @@ htp_status_t htp_connp_RES_BODY_DETERMINE(htp_connp_t *connp) {
         return HTP_OK;
     }
 
+    // A request can indicate it waits for headers validation
+    // before sending its body cf
+    // https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Expect
+    if (connp->out_tx->response_status_number >= 400 &&
+        connp->out_tx->response_status_number <= 499 &&
+        connp->in_content_length > 0 &&
+        connp->in_body_data_left == connp->in_content_length) {
+        htp_header_t *exp = htp_table_get_c(connp->out_tx->request_headers, "expect");
+        if ((exp != NULL) && (bstr_cmp_c_nocase(exp->value, "100-continue") == 0)) {
+            connp->in_state = htp_connp_REQ_FINALIZE;
+        }
+    }
+
     // 1. Any response message which MUST NOT include a message-body
     //  (such as the 1xx, 204, and 304 responses and any response to a HEAD
     //  request) is always terminated by the first empty line after the
@@ -1228,7 +1259,7 @@ int htp_connp_res_data(htp_connp_t *connp, const htp_time_t *timestamp, const vo
     // only if the stream has been closed. We do not allow zero-sized
     // chunks in the API, but we use it internally to force the parsers
     // to finalize parsing.
-    if (((data == NULL) || (len == 0)) && (connp->out_status != HTP_STREAM_CLOSED)) {
+    if (len == 0 && connp->out_status != HTP_STREAM_CLOSED) {
         htp_log(connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0, "Zero-length data chunks are not allowed");
 
         #ifdef HTP_DEBUG
@@ -1277,7 +1308,22 @@ int htp_connp_res_data(htp_connp_t *connp, const htp_time_t *timestamp, const vo
         // or if we've run out of data. We are relying
         // on processors to add error messages, so we'll
         // keep quiet here.
-        htp_status_t rc = connp->out_state(connp);
+        htp_status_t rc;
+
+        //handle gap
+        if (data == NULL && len > 0) {
+            if (connp->out_state == htp_connp_RES_BODY_IDENTITY_CL_KNOWN ||
+                connp->out_state == htp_connp_RES_BODY_IDENTITY_STREAM_CLOSE) {
+                rc = connp->out_state(connp);
+            } else if (connp->out_state == htp_connp_RES_FINALIZE) {
+                rc = htp_tx_state_response_complete_ex(connp->out_tx, 0);
+            } else {
+                htp_log(connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0, "Gaps are not allowed during this state");
+                return HTP_STREAM_CLOSED;
+            }
+        } else {
+            rc = connp->out_state(connp);
+        }
         if (rc == HTP_OK) {
             if (connp->out_status == HTP_STREAM_TUNNEL) {
                 #ifdef HTP_DEBUG
